(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5905,        175]*)
(*NotebookOutlinePosition[      6602,        199]*)
(*  CellTagsIndexPosition[      6558,        195]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Spectral Portraits", "Subtitle"],

Cell["\<\
Spectral portraits represent a visual way to analyze the sensitivity of the \
eigenvalues of matrices to perturbations.  For mor information on spectral \
portraits, visit the NIST site:
http://math.nist.gov/MatrixMarket/spectral.html\
\>", "Text"],

Cell[BoxData[
    \(\(SetDirectory[
        ToFileName[\(("\<FileName\>" /. 
                NotebookInformation[
                  EvaluationNotebook[]])\)[\([1]\)]]];\)\)], "Input"],

Cell[BoxData[
    \(\(a = Import["\<west0655.mtx\>", "\<MatrixMarket\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(a = Normal[a];\)\)], "Input"],

Cell[BoxData[
    \(\(norma = Norm[N[a]];\)\)], "Input"],

Cell[TextData[{
  "Definition of spectral portrait function: ",
  Cell[BoxData[
      \(TraditionalForm\`z\[LongRightArrow]\(spp(
            z)\) = \(log\_10\) \(\[LeftDoubleBracketingBar]\((A - z\ \
I)\)\^\(-1\)\[RightDoubleBracketingBar]\_2\) \[LeftDoubleBracketingBar]A\
\[RightDoubleBracketingBar]\_2\)]]
}], "Text"],

Cell[BoxData[
    \(SpectralPortrait[a_, z_] := 
      Module[{s = Length[a]}, \[IndentingNewLine]Log[10, 
            Norm[Inverse[z\ IdentityMatrix[s] - a]]] norma]\)], "Input"],

Cell[CellGroupData[{

Cell["Single CPU Computation", "Subsection"],

Cell["\<\
The calculation time of the spectral portrait varies with the step size. \
Small stepsizes can be quite time consuming.  Some approximate step size \
timings to give you an example of how the time scales with stepsize are: 50 \
-> 200 sec, 20-> 500 sec.\
\>", "Text"],

Cell[BoxData[
    \(AbsoluteTiming[\(t = 
          Table[SpectralPortrait[N[a], N[x + y*I]], {x, \(-100\), 100, 
              50}, {y, \(-100\), 100, 50}];\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parallel Computation", "Subsection"],

Cell[TextData[{
  "Spectral portraits lend themselves very well to parallel computations.  To \
speed up the above code, one can make use of ",
  Cell[BoxData[
      FormBox[
        RowBox[{"grid", 
          StyleBox[
            AdjustmentBox["Mathematica",
              BoxMargins->{{-0.175, 0}, {0, 0}}],
            FontSlant->"Italic"]}], TextForm]]],
  " and/or the Parallel Computing Toolkit (PCT) to perform a ParallelTable \
instead.  This parallel implementation scales very well since each table \
element is independent from the others, making this an embarrassingly \
parallel computation.  If you have PCT, you can load the package and launch a \
number of slave nodes to speed up the calculation.  With the exception of the \
initialization steps whicj are not included here (loading the package, \
launching the slaves, and evaluating the functions on the slaves), the only \
change to the code is as follows."
}], "Text"],

Cell[TextData[{
  "*Note* - This code only works with PCT and/or ",
  Cell[BoxData[
      FormBox[
        RowBox[{"grid", 
          StyleBox[
            AdjustmentBox["Mathematica",
              BoxMargins->{{-0.175, 0}, {0, 0}}],
            FontSlant->"Italic"]}], TextForm]]],
  ".  If you don't have this package, use the single CPU code in the previous \
section."
}], "Text"],

Cell[BoxData[
    \(AbsoluteTiming[\(t = 
          ParallelTable[
            SpectralPortrait[N[a], N[x + y*I]], {x, \(-100\), 100, 
              20}, {y, \(-100\), 100, 20}];\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plotting the portrait", "Subsection"],

Cell[BoxData[
    \(\(heat[z_] := 
        RGBColor[E^\(-\((\((z - 1)\)\^2/ .3)\)\), 
          E^\(-\((\((z - 1)\)\^2/ .1)\)\), 
          E^\(-\((\((z - 1)\)\^2/ .001)\)\)];\)\)], "Input"],

Cell[BoxData[
    \(\(myresult = 
        ListContourPlot[Transpose[t], ColorFunction \[Rule] heat, 
          Frame \[Rule] True, 
          FrameLabel \[Rule] {"\<Real\>", "\<Imaginary\>"}, 
          ImageSize \[Rule] 550, 
          FrameTicks \[Rule] {{{1, \(-100\)}, {3.5, \(-50\)}, {6, 0}, {8.5, 
                  50}, {11, 100}}, {{1, \(-100\)}, {3.5, \(-50\)}, {6, 
                  0}, {8.5, 50}, {11, 100}}}, Contours \[Rule] 20, 
          ContourLines \[Rule] False];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 935}},
WindowSize->{1091, 872},
WindowMargins->{{61, Automatic}, {0, Automatic}},
Magnification->1,
StyleDefinitions -> "Default.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 38, 0, 51, "Subtitle"],
Cell[1817, 55, 258, 5, 52, "Text"],
Cell[2078, 62, 183, 4, 30, "Input"],
Cell[2264, 68, 88, 1, 30, "Input"],
Cell[2355, 71, 51, 1, 30, "Input"],
Cell[2409, 74, 56, 1, 30, "Input"],
Cell[2468, 77, 321, 7, 33, "Text"],
Cell[2792, 86, 179, 3, 50, "Input"],

Cell[CellGroupData[{
Cell[2996, 93, 44, 0, 38, "Subsection"],
Cell[3043, 95, 277, 5, 52, "Text"],
Cell[3323, 102, 173, 3, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3533, 110, 42, 0, 38, "Subsection"],
Cell[3578, 112, 941, 18, 90, "Text"],
Cell[4522, 132, 385, 11, 33, "Text"],
Cell[4910, 145, 194, 4, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5141, 154, 43, 0, 38, "Subsection"],
Cell[5187, 156, 190, 4, 31, "Input"],
Cell[5380, 162, 497, 9, 70, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

